<?php

namespace Eniture\StandardBoxSizes\Controller\FedExOneRate;

use \Magento\Framework\App\Action\Action;

class OneRateSaveBoxes extends Action
{
    /**
     * @var \Magento\Framework\DB\Adapter\AdapterInterface
     */
    private $connection;
    /**
     * @var string
     */
    private $boxSizesTable;
    /**
     * @var \Eniture\StandardBoxSizes\Helper\Data
     */
    private $dataHelper;
    /**
     * @var object
     */
    private $boxsizesFactory;
    /**
     * OneRateSaveBoxes constructor.
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param \Eniture\StandardBoxSizes\Helper\Data $dataHelper
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\App\ResourceConnection $resource,
        \Eniture\StandardBoxSizes\Helper\Data $dataHelper,
        \Eniture\StandardBoxSizes\Model\BoxsizesFactory $boxsizesFactory
    ) {
        $this->connection        = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        $this->boxSizesTable     = $resource->getTableName('boxsizes');
        $this->dataHelper        = $dataHelper;
        $this->boxsizesFactory = $boxsizesFactory->create();
        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface|void
     */
    function execute()
    {
        $postdata = $this->getRequest()->getPost();

        $actualData = $this->dataHelper->oneRateData();
        $mainJsonArrays = $old = [];

        $collection = $this->boxsizesFactory->getCollection()->addFilter('type', ['eq' => 'onerate']);
        foreach ($collection as $col){
            $old[] = $col->getData();
        }

        $old = array_column($old, 'oneRateId');
        $old = array_flip($old);

        if (isset($postdata['fedExOneRate']) || !empty($old)) {
            $fedexOneRateData = (isset($postdata['fedExOneRate'])) ? $postdata['fedExOneRate'] : [];
            foreach ($fedexOneRateData as $key => $oneRate) {
                if (!isset($old[$oneRate])) {
                    $fedExOneRate = $this->fedExOneRateMakeArray($actualData[$oneRate], $oneRate);

                    extract($fedExOneRate);
                    if (empty($getoneRateData)) {
                        $insertQry = $this->connection->insert("$this->boxSizesTable", $fedExOneRate);

                        $lastid = $this->connection->lastInsertId();

                        $available = ($boxavailable == 1)?'Yes':'No';
                        $oneRateList = [
                            'nickname'      => $boxnickname,
                            'boxlength'     => $boxlength,
                            'boxwidth'      => $boxwidth,
                            'boxheight'     => $boxheight,
                            'boxmaxweight'  => $boxmaxweight,
                            'boxweight'     => $boxweight,
                            'boxavailable'  => $available,
                            'id'            => $lastid,
                            'insertid'      => $insertQry,
                            'oneRateId'     => $oneRateId,
                        ];

                        $mainJsonArrays[$key] = $oneRateList;
                    }
                } else {
                    unset($old[$oneRate]);
                }
            }
        }
        $delete = [];
        foreach ($old as $key => $value) {
            $boxColl = $this->boxsizesFactory->getCollection()
                ->addFilter('oneRateId', ['eq' => $key])
                ->addFilter('type', ['eq' => 'onerate']);

            foreach ($boxColl as $box){
                $box_id[] = $box->getData();
            }

            $box_id = (isset($box_id[0]['box_id'])) ? $box_id[0]['box_id'] : "";
            $delete[] = $box_id;
            $this->connection->delete("$this->boxSizesTable", "box_id='".$box_id."'");
        }

        $response = ['data' => $mainJsonArrays , 'delete' => $delete];

        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody(json_encode($response));
    }

    /**
     * @param $data
     * @param $id
     * @return array
     */
    function fedExOneRateMakeArray($data, $id)
    {
        return [
            'boxlength'         => $data[0],
            'boxwidth'          => $data[1],
            'boxheight'         => $data[2],
            'boxmaxweight'      => $data[3],
            'boxweight'         => $data[4],
            'boxavailable'      => '1',
            'boxnickname'       => $data[7],
            'fedex_box_type'    => $data[5],
            'type'              => 'onerate',
            'oneRateId'         => $id,
        ];
    }
}
