<?php

/**
 * Eniture Standard Box Sizes
 * @package     Eniture Standard Box Sizes
 * @author      Eniture-Technology
 */
namespace Eniture\StandardBoxSizes\Setup;

use Magento\Eav\Setup\EavSetup;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\InstallDataInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\DB\Ddl\Table;

/**
 * Class InstallData
 * @package Eniture\StandardBoxSizes\Setup
 */
class InstallData implements InstallDataInterface
{
    /**
     * EAV setup factory
     * @var EavSetupFactory
     */
    private $eavSetupFactory;
    /**
     * ResourceConnection
     * @var resource
     */
    private $resource;
    /**
     * Resource Connection
     * @var connection
     */
    private $connection;
    /**
     * State
     * @var state
     */
    private $state;
    /**
     * Config Resource Interface
     * @var resourceConfig
     */
    private $resourceConfig;
    /**
     * Table names
     * @var _tableNames
     */
    private $_tableNames;

    /**
     * InstallData constructor.
     * @param EavSetupFactory $eavSetupFactory
     * @param \Eniture\StandardBoxSizes\App\State $state
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param \Magento\Framework\App\Config\ConfigResource\ConfigInterface $resourceConfig
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    function __construct(
        EavSetupFactory $eavSetupFactory,
        \Eniture\StandardBoxSizes\App\State $state,
        \Magento\Framework\App\ResourceConnection $resource,
        \Magento\Framework\App\Config\ConfigResource\ConfigInterface $resourceConfig
    ) {

        $this->eavSetupFactory      = $eavSetupFactory;
        $this->resource            = $resource;
        $this->connection          = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        $this->state                = $state;
        $this->resourceConfig       = $resourceConfig;
    }

    /**
     * @param ModuleDataSetupInterface $setup
     * @param ModuleContextInterface $context
     */
    public function install(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        $this->state->validateAreaCode();
        $installer = $setup;
        $installer->startSetup();

        $this->saveConfigurations('binBox/trial/use', '0');

        /** @var EavSetup $eavSetup */
        $eavSetup = $this->eavSetupFactory->create(['setup' => $installer]);

        $this->getTableNames();
        $this->createBoxSizeOrderAttr($installer);
        $this->createBoxSizeProductAttr($installer, $eavSetup);
        $this->createBoxSizeTable($installer);

        $this->saveConfigurations('binPackaging/suspend/value', 'no');
        $installer->endSetup();
    }

    /**
     * @param $path
     * @param $value
     */
    function saveConfigurations($path, $value)
    {
        $this->resourceConfig->saveConfig(
            $path,
            $value,
            \Magento\Framework\App\Config\ScopeConfigInterface::SCOPE_TYPE_DEFAULT,
            \Magento\Store\Model\Store::DEFAULT_STORE_ID
        );
    }

    /**
     * Set Attribute names globally
     */
    function getTableNames()
    {
        $this->_tableNames = [
                'eav_attribute' => $this->resource->getTableName('eav_attribute'),
            ];
    }

    /**
     * @param $installer
     */
    function createBoxSizeOrderAttr($installer)
    {
        $installer->getConnection()->addColumn(
            $installer->getTable('sales_order'),
            'bin_packaging',
            [
                'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                'nullable' => true,
                'default'   => '',
                'length' => '2M',
                'comment' => 'Bin Packaging Data'
            ]
        );
    }

    /**
     * @param $installer
     * @param $eavSetup
     */
    function createBoxSizeProductAttr($installer, $eavSetup)
    {
        $isVerticalRotationExist = $this->connection->fetchOne("SELECT count(*) AS count FROM ".$this->_tableNames['eav_attribute']." WHERE attribute_code = 'en_vertical_rotation'");
        if ($isVerticalRotationExist == 0) {
            $this->getAttributeArray($eavSetup, 'en_vertical_rotation', 'int', 'Allow vertical rotation', 'select', 'Magento\Eav\Model\Entity\Attribute\Source\Boolean', 83);
        }

        $isShipOwnPackageExist = $this->connection->fetchOne("SELECT count(*) AS count FROM ".$this->_tableNames['eav_attribute']." WHERE attribute_code = 'en_own_package'");
        if ($isShipOwnPackageExist == 0) {
            $this->getAttributeArray($eavSetup, 'en_own_package', 'int', 'Ship as own package', 'select', 'Magento\Eav\Model\Entity\Attribute\Source\Boolean', 84);
        }

        $installer->endSetup();
    }

    /**
     * @param $eavSetup
     * @param $code
     * @param $type
     * @param $label
     * @param $input
     * @param $source
     * @param $order
     * @return mixed
     */
    function getAttributeArray($eavSetup, $code, $type, $label, $input, $source, $order)
    {
        $attrArr = $eavSetup->addAttribute(
            \Magento\Catalog\Model\Product::ENTITY,
            $code,
            [
                    'group'                 => 'Product Details',
                    'type'                  => $type,
                    'backend'               => '',
                    'frontend'              => '',
                    'label'                 => $label,
                    'input'                 => $input,
                    'class'                 => '',
                    'source'                => $source,
                     'global'               => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_STORE,
                    'required'              => false,
                    'visible_on_front'      => false,
                    'is_configurable'       => true,
                    'sort_order'            => $order,
                    'user_defined'          => true,
                    'default'               => '0'
            ]
        );

        return $attrArr;
    }

    /**
     * @param $installer
     */

    function createBoxSizeTable($installer)
    {
        $tableName = $installer->getTable('boxsizes');
        if ($installer->getConnection()->isTableExists($tableName) != true) {
            $table = $installer->getConnection()
                ->newTable($tableName)
                ->addColumn('box_id', Table::TYPE_INTEGER, null, [
                    'identity'  => true,
                    'unsigned'  => true,
                    'nullable'  => false,
                    'primary'   => true,
                    ], 'Id')
                ->addColumn('boxnickname', Table::TYPE_TEXT, 200, [
                    'nullable'  => false,
                    ], 'boxnickname')
                ->addColumn('boxlength', Table::TYPE_DECIMAL, '6,3', [
                    ], 'boxlength')
                ->addColumn('boxwidth', Table::TYPE_DECIMAL, '6,3', [
                        ], 'boxwidth')
                ->addColumn('boxheight', Table::TYPE_DECIMAL, '6,3', [
                        ], 'boxheight')
                ->addColumn('boxmaxweight', Table::TYPE_DECIMAL, '6,3', [
                        ], 'boxmaxweight')
                ->addColumn('boxweight', Table::TYPE_DECIMAL, '6,3', [
                        ], 'boxweight')
                ->addColumn('boxfee', Table::TYPE_TEXT, 200, [
                        ], 'boxfee')
                ->addColumn('oneRateId', Table::TYPE_INTEGER, null, [
                        ], 'oneRateId')
                ->addColumn('fedex_box_type', Table::TYPE_TEXT, 200, [
                        'nullable'  => false,
                        ], 'fedex_box_type')
                ->addColumn('type', Table::TYPE_TEXT, 200, [
                        'nullable'  => false,
                        ], 'type')
                ->addColumn('boxavailable', Table::TYPE_BOOLEAN, 1, [
                        'nullable'  => false,
                        ], 'boxavailable');
            $installer->getConnection()->createTable($table);
        }
        $installer->endSetup();
    }
}
